@@ -1,6 +1,6 @@
 /*
    CheMPS2: a spin-adapted implementation of DMRG for ab initio quantum chemistry
-   Copyright (C) 2013-2015 Sebastian Wouters
+   Copyright (C) 2013-2016 Sebastian Wouters
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,27 +24,31 @@
 #include <algorithm>
 
 #include "EdmistonRuedenberg.h"
-#include "DMRGSCFVmatRotations.h"
+#include "DMRGSCFrotations.h"
 #include "Lapack.h"
 
 using std::cout;
 using std::endl;
 using std::max;
 
-CheMPS2::EdmistonRuedenberg::EdmistonRuedenberg(Hamiltonian * HamIn, const int printLevelIn){
+CheMPS2::EdmistonRuedenberg::EdmistonRuedenberg( const FourIndex * Vmat, const int group, const int printLevelIn ){
 
-   Ham = HamIn;
+   VMAT_ORIG = Vmat;
    printLevel = printLevelIn;
-   SymmInfo.setGroup(Ham->getNGroup());
+   SymmInfo.setGroup( group );
    
-   int * Isizes = new int[SymmInfo.getNumberOfIrreps()];
-   int * Zeroes = new int[SymmInfo.getNumberOfIrreps()];
-   for (int irrep=0; irrep<SymmInfo.getNumberOfIrreps(); irrep++){ Isizes[irrep] = Zeroes[irrep] = 0; }
-   for (int orb=0; orb<Ham->getL(); orb++){ Isizes[Ham->getOrbitalIrrep(orb)]++; }
+   int * Isizes = new int[ SymmInfo.getNumberOfIrreps() ];
+   int * Zeroes = new int[ SymmInfo.getNumberOfIrreps() ];
+   int L = 0;
+   for ( int irrep = 0; irrep < SymmInfo.getNumberOfIrreps(); irrep++ ){
+      Isizes[ irrep ] = VMAT_ORIG->get_irrep_size( irrep );
+      Zeroes[ irrep ] = 0;
+      L += Isizes[ irrep ];
+   }
    
-   iHandler = new DMRGSCFindices(Ham->getL(), Ham->getNGroup(), Zeroes, Isizes, Zeroes); //Supposes all orbitals are active
-   unitary  = new DMRGSCFunitary(iHandler);
-   VmatRotated = new FourIndex(Ham->getNGroup(), Isizes);
+   iHandler = new DMRGSCFindices( L, group, Zeroes, Isizes, Zeroes ); //Supposes all orbitals are active
+   unitary  = new DMRGSCFunitary( iHandler );
+   VmatRotated = new FourIndex( group, Isizes );
    
    delete [] Zeroes;
    delete [] Isizes;
@@ -62,24 +66,14 @@ CheMPS2::EdmistonRuedenberg::~EdmistonRuedenberg(){
 double CheMPS2::EdmistonRuedenberg::Optimize(double * temp1, double * temp2, const bool startFromRandomUnitary, const double gradThreshold, const int maxIter){
 
    //Clear the unitary
-   for (int irrep=0; irrep<SymmInfo.getNumberOfIrreps(); irrep++){
-      const int linsize = iHandler->getNORB(irrep);
-      double * block = unitary->getBlock(irrep);
-      for (int cnt=0; cnt<linsize*linsize; cnt++){ block[cnt] = 0.0; }
-      for (int cnt=0; cnt<linsize; cnt++){ block[cnt*(1+linsize)] = 1.0; }
-   }
-   
+   unitary->identity();
+
    //Setting up the variables for the gradient
    double gradNorm = 1.0;
-   int numVariables = 0;
-   for (int irrep=0; irrep<SymmInfo.getNumberOfIrreps(); irrep++){
-      if (iHandler->getNORB(irrep) > 1){
-         numVariables += iHandler->getNORB(irrep) * (iHandler->getNORB(irrep) - 1) / 2;
-      }
-   }
+   const int numVariables = iHandler->getROTparamsize();
    double * gradient = new double[numVariables];
    for (int cnt=0; cnt<numVariables; cnt++){ gradient[cnt] = 0.0; }
-   
+
    //Randomize the unitary if asked
    if (startFromRandomUnitary){
       for (int cnt=0; cnt<numVariables; cnt++){ gradient[cnt] = ((double) rand())/RAND_MAX - 0.5; }
@@ -87,8 +81,8 @@ double CheMPS2::EdmistonRuedenberg::Optimize(double * temp1, double * temp2, con
       for (int cnt=0; cnt<numVariables; cnt++){ gradient[cnt] = 0.0; }
    }
 
-   DMRGSCFVmatRotations theRotator(Ham, iHandler);
-   theRotator.fillVmatRotated(VmatRotated, unitary, temp1, temp2);
+   const int mem_size = iHandler->getL() * iHandler->getL() * iHandler->getL() * iHandler->getL();
+   DMRGSCFrotations::rotate( VMAT_ORIG, VmatRotated, NULL, 'F', 'F', 'F', 'F', iHandler, unitary, temp1, temp2, mem_size, "edmistonruedenberg" );
 
    //Setting up the variables for the cost function
    double Icost = costFunction();
@@ -106,7 +100,7 @@ double CheMPS2::EdmistonRuedenberg::Optimize(double * temp1, double * temp2, con
    
       //Rotate the Vmat
       Icost_previous = Icost;
-      theRotator.fillVmatRotated(VmatRotated, unitary, temp1, temp2);
+      DMRGSCFrotations::rotate( VMAT_ORIG, VmatRotated, NULL, 'F', 'F', 'F', 'F', iHandler, unitary, temp1, temp2, mem_size, "edmistonruedenberg" );
       Icost = costFunction();
       
       /* What if the cost function has dimished? Then make the rotation step a bit smaller!
@@ -120,7 +114,7 @@ double CheMPS2::EdmistonRuedenberg::Optimize(double * temp1, double * temp2, con
             nIterationsBACK++;
             for (int cnt=0; cnt<numVariables; cnt++){ gradient[cnt] *= 0.5; }
             unitary->updateUnitary(temp1, temp2, gradient, true, false); //multiply = true; compact = false
-            theRotator.fillVmatRotated(VmatRotated, unitary, temp1, temp2);
+            DMRGSCFrotations::rotate( VMAT_ORIG, VmatRotated, NULL, 'F', 'F', 'F', 'F', iHandler, unitary, temp1, temp2, mem_size, "edmistonruedenberg" );
             Icost = costFunction();
          }
          if (printLevel>1){ cout << "                                     WARNING : Rotated back a bit. Now Icost = " << Icost << endl; }
@@ -374,6 +368,9 @@ void CheMPS2::EdmistonRuedenberg::FiedlerExchange(const int maxlinsize, double *
 
    //For information on the Fiedler vector: see http://web.eecs.utk.edu/~mberry/order/node9.html
 
+   const int mem_size = iHandler->getL() * iHandler->getL() * iHandler->getL() * iHandler->getL();
+   DMRGSCFrotations::rotate( VMAT_ORIG, VmatRotated, NULL, 'F', 'F', 'F', 'F', iHandler, unitary, temp1, temp2, mem_size, "edmistonruedenberg" );
+
    if (printLevel>0){ cout << "   EdmistonRuedenberg::FiedlerExchange : Cost function at start = " << FiedlerExchangeCost() << endl; }
    
    int * reorder = new int[maxlinsize];
@@ -405,10 +402,9 @@ void CheMPS2::EdmistonRuedenberg::FiedlerExchange(const int maxlinsize, double *
    }
    
    delete [] reorder;
-   
-   DMRGSCFVmatRotations theRotator(Ham, iHandler);
-   theRotator.fillVmatRotated(VmatRotated, unitary, temp1, temp2);
-   
+
+   DMRGSCFrotations::rotate( VMAT_ORIG, VmatRotated, NULL, 'F', 'F', 'F', 'F', iHandler, unitary, temp1, temp2, mem_size, "edmistonruedenberg" );
+
    if (printLevel>0){ cout << "   EdmistonRuedenberg::FiedlerExchange : Cost function at end   = " << FiedlerExchangeCost() << endl; }
 
 }
