@@ -1,6 +1,6 @@
 /*
    CheMPS2: a spin-adapted implementation of DMRG for ab initio quantum chemistry
-   Copyright (C) 2013-2015 Sebastian Wouters
+   Copyright (C) 2013-2016 Sebastian Wouters
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,16 +20,18 @@
 #ifndef CASSCF_CHEMPS2_H
 #define CASSCF_CHEMPS2_H
 
+#include <string>
+
 #include "Hamiltonian.h"
 #include "Irreps.h"
 #include "TwoDM.h"
+#include "ThreeDM.h"
 #include "DMRG.h"
 #include "Problem.h"
 #include "Options.h"
 #include "ConvergenceScheme.h"
 #include "DMRGSCFindices.h"
 #include "DMRGSCFunitary.h"
-#include "DIIS.h"
 #include "DMRGSCFoptions.h"
 #include "DMRGSCFwtilde.h"
 #include "DMRGSCFmatrix.h"
@@ -164,44 +166,55 @@ namespace CheMPS2{
       public:
       
          //! Constructor
-         /** \param filename The file containing the Psi4 Hamiltonian text dump (mointegrals.so_PRINT) output to start the DMRGSCF calculations. */
-         CASSCF(const string filename);
-         
-         //! Constructor
-         /** \param HamIn Hamiltonian containing the matrix elements of the Hamiltonian for which a CASSCF calculation is desired
-             \param DOCCin Array containing the number of doubly occupied HF orbitals per irrep
-             \param SOCCin Array containing the number of singly occupied HF orbitals per irrep */
-         CASSCF(Hamiltonian * HamIn, int * DOCCin, int * SOCCin);
+         /** \param ham_in Hamiltonian containing the matrix elements of the Hamiltonian for which a CASSCF calculation is desired
+             \param docc  Array containing the number of doubly occupied HF orbitals per irrep
+             \param socc  Array containing the number of singly occupied HF orbitals per irrep
+             \param nocc  Array containing the number of doubly occupied (inactive) orbitals per irrep
+             \param ndmrg Array containing the number of active orbitals per irrep
+             \param nvirt Array containing the number of virtual (secondary) orbitals per irrep
+             \param tmp_folder Temporary work folder for the DMRG renormalized operators and the ERI rotations */
+         CASSCF( Hamiltonian * ham_in, int * docc, int * socc, int * nocc, int * ndmrg, int * nvirt, const string tmp_folder=CheMPS2::defaultTMPpath );
          
          //! Destructor
          virtual ~CASSCF();
          
          //! Get the number of irreps
          /** \return The number of irreps */
-         int getNumberOfIrreps();
+         int get_num_irreps();
          
-         //! Set the start of the CASSCF calculation
-         /** \param NoccIn Array of length numberOfIrreps containing the number of double occupied HF orbitals per irrep for the CASSCF loop.
-             \param NDMRGIn Array of length numberOfIrreps containing the number of active orbitals per irrep for the CASSCF loop.
-             \param NvirtIn Array of length numberOfIrreps containing the number of empty orbitals per irrep for the CASSCF loop. */
-         void setupStart(int * NoccIn, int * NDMRGIn, int * NvirtIn);
-         
-         //! Does the state-specific CASSCF cycle with the augmented Hessian Newton-Raphson method
+         //! Do the CASSCF cycles with the augmented Hessian Newton-Raphson method
          /** \param Nelectrons Total number of electrons in the system: occupied HF orbitals + active space
              \param TwoS Twice the targeted spin
              \param Irrep Desired wave-function irrep
-             \param OptScheme The optimization scheme to run the inner DMRG loop
+             \param OptScheme The optimization scheme to run the inner DMRG loop. If NULL: use FCI instead of DMRG.
              \param rootNum Denotes the targeted state in state-specific CASSCF; 1 means ground state, 2 first excited state etc.
-             \param theDMRGSCFoptions Contains the DMRGSCF options
+             \param scf_options Contains the DMRGSCF options
              \return The converged DMRGSCF energy */
-         double doCASSCFnewtonraphson(const int Nelectrons, const int TwoS, const int Irrep, ConvergenceScheme * OptScheme, const int rootNum, DMRGSCFoptions * theDMRGSCFoptions);
-         
+         double solve( const int Nelectrons, const int TwoS, const int Irrep, ConvergenceScheme * OptScheme, const int rootNum, DMRGSCFoptions * scf_options );
+
+         //! Calculate the caspt2 correction energy for a converged casscf wavefunction
+         /** \param Nelectrons Total number of electrons in the system: occupied HF orbitals + active space
+             \param TwoS Twice the targeted spin
+             \param Irrep Desired wave-function irrep
+             \param OptScheme The optimization scheme to run the inner DMRG loop. If NULL: use FCI instead of DMRG.
+             \param rootNum Denotes the targeted state in state-specific CASSCF; 1 means ground state, 2 first excited state etc.
+             \param scf_options Contains the DMRGSCF options
+             \param IPEA The CASPT2 IPEA shift from Ghigo, Roos and Malmqvist, Chemical Physics Letters 396, 142-149 (2004)
+             \param IMAG The CASPT2 imaginary shift from Forsberg and Malmqvist, Chemical Physics Letters 274, 196-204 (1997)
+             \param PSEUDOCANONICAL If true, use the exact DMRG 4-RDM in the pseudocanonical basis. If false, use the cumulant approximated DMRG 4-RDM in the unrotated basis.
+             \param CHECKPOINT If true, write checkpoints to disk and read them back in again in order to perform the contraction of the generalized Fock operator with the 4-RDM in multiple runs.
+             \param CUMULANT If true, a cumulant approximation is used for the 4-RDM and CHECKPOINT is overwritten to false. If false, the full 4-RDM is used.
+             \return The CASPT2 variational correction energy */
+         double caspt2( const int Nelectrons, const int TwoS, const int Irrep, ConvergenceScheme * OptScheme, const int rootNum, DMRGSCFoptions * scf_options, const double IPEA, const double IMAG, const bool PSEUDOCANONICAL, const bool CHECKPOINT = false, const bool CUMULANT = false );
+
          //! CASSCF unitary rotation remove call
-         void deleteStoredUnitary(const string filename=CheMPS2::DMRGSCF_unitaryStorageName){ unitary->deleteStoredUnitary(filename); }
-         
+         /* \param filename File to delete */
+         static void deleteStoredUnitary( const string filename=CheMPS2::DMRGSCF_unitary_storage_name ){ delete_file( filename ); }
+
          //! CASSCF DIIS vectors remove call
-         void deleteStoredDIIS(const string filename=CheMPS2::DMRGSCF_DIISstorageName){ if (theDIIS!=NULL){ theDIIS->deleteStoredDIIS(filename); }}
-         
+         /* \param filename File to delete */
+         static void deleteStoredDIIS( const string filename=CheMPS2::DMRGSCF_diis_storage_name ){ delete_file( filename ); }
+
          //! Build the F-matrix (Eq. (11) in the Siegbahn paper [CAS3])
          /** \param localFmat Matrix where the result should be stored
              \param localTmat Matrix which contains the one-electron integrals
@@ -211,8 +224,8 @@ namespace CheMPS2{
              \param theInts The rotated two-electron integrals (at most 2 virtual indices)
              \param local2DM The DMRG 2-RDM
              \param local1DM The DMRG 1-RDM */
-         static void buildFmat(DMRGSCFmatrix * localFmat, const DMRGSCFmatrix * localTmat, const DMRGSCFmatrix * localJKocc, const DMRGSCFmatrix * localJKact, const DMRGSCFindices * localIdx, const DMRGSCFintegrals * theInts, double * local2DM, double * local1DM);
-         
+         static void buildFmat( DMRGSCFmatrix * localFmat, const DMRGSCFmatrix * localTmat, const DMRGSCFmatrix * localJKocc, const DMRGSCFmatrix * localJKact, const DMRGSCFindices * localIdx, const DMRGSCFintegrals * theInts, double * local2DM, double * local1DM );
+
          //! Build the Wtilde-matrix (Eq. (20b) in the Siegbahn paper [CAS3])
          /** \param localwtilde Where the result should be stored
              \param localTmat Matrix which contains the one-electron integrals
@@ -222,8 +235,8 @@ namespace CheMPS2{
              \param theInts The rotated two-electron integrals (at most 2 virtual indices)
              \param local2DM The DMRG 2-RDM
              \param local1DM The DMRG 1-RDM */
-         static void buildWtilde(DMRGSCFwtilde * localwtilde, const DMRGSCFmatrix * localTmat, const DMRGSCFmatrix * localJKocc, const DMRGSCFmatrix * localJKact, const DMRGSCFindices * localIdx, const DMRGSCFintegrals * theInts, double * local2DM, double * local1DM);
-         
+         static void buildWtilde( DMRGSCFwtilde * localwtilde, const DMRGSCFmatrix * localTmat, const DMRGSCFmatrix * localJKocc, const DMRGSCFmatrix * localJKact, const DMRGSCFindices * localIdx, const DMRGSCFintegrals * theInts, double * local2DM, double * local1DM );
+
          //! Calculate the augmented Hessian Newton-Raphson update for the orthogonal orbital rotation matrix
          /** \param localFmat Matrix which contains the Fock operator (Eq. (11) in the Siegbahn paper [CAS3])
              \param localwtilde Object which contains the second order derivative of the energy with respect to the unitary (Eq. (20b) in the Siegbahn paper [CAS3])
@@ -232,129 +245,163 @@ namespace CheMPS2{
              \param theupdate Where the augmented Hessian Newton-Raphson update will be stored
              \param updateNorm Pointer to one double to store the update norm
              \param gradNorm Pointer to one double to store the gradient norm */
-         static void augmentedHessianNR(const DMRGSCFmatrix * localFmat, const DMRGSCFwtilde * localwtilde, const DMRGSCFindices * localIdx, const DMRGSCFunitary * localUmat, double * theupdate, double * updateNorm, double * gradNorm);
-         
+         static void augmentedHessianNR( DMRGSCFmatrix * localFmat, DMRGSCFwtilde * localwtilde, const DMRGSCFindices * localIdx, const DMRGSCFunitary * localUmat, double * theupdate, double * updateNorm, double * gradNorm );
+
          //! Copy over the DMRG 2-RDM
          /** \param theDMRG2DM The 2-RDM from the DMRG object
-             \param totOrbDMRG The total number of DMRG orbitals
-             \param localDMRG2DM The CASSCF 2-RDM */
-         static void copy2DMover(TwoDM * theDMRG2DM, const int totOrbDMRG, double * localDMRG2DM);
-         
+             \param LAS The total number of DMRG orbitals
+             \param two_dm The CASSCF 2-RDM */
+         static void copy2DMover( TwoDM * theDMRG2DM, const int LAS, double * two_dm );
+
          //! Construct the 1-RDM from the 2-RDM
-         /** \param nDMRGelectrons The number of DMRG active space electrons
-             \param totOrbDMRG The total number of DMRG orbitals
-             \param localDMRG1DM The CASSCF 1-RDM
-             \param localDMRG2DM The CASSCF 2-RDM */
-         static void setDMRG1DM(const int nDMRGelectrons, const int totOrbDMRG, double * localDMRG1DM, double * localDMRG2DM);
-         
-         //! Calculate the natural orbitals and their occupation numbers
-         /** \param localIdx Object which handles the index conventions for CASSCF
-             \param eigenvecs Where to store the natural orbitals
-             \param workmem Work memory
-             \param localDMRG1DM The CASSCF 1-RDM */
-         static void calcNOON(CheMPS2::DMRGSCFindices * localIdx, double * eigenvecs, double * workmem, double * localDMRG1DM);
-         
-         //! Rotate the CASSCF 1-RDM and 2-RDM to a new basis (to calculate the gradient and Hessian)
-         /** \param nDMRGelectrons The number of DMRG active space electrons
-             \param totOrbDMRG The total number of DMRG orbitals
-             \param eigenvecs Where the eigenvectors are stored
-             \param work Work memory
-             \param localDMRG1DM The CASSCF 1-RDM
-             \param localDMRG2DM The CASSCF 2-RDM */
-         static void rotate2DMand1DM(const int nDMRGelectrons, int totOrbDMRG, double * eigenvecs, double * work, double * localDMRG1DM, double * localDMRG2DM);
-         
+         /** \param num_elec The number of DMRG active space electrons
+             \param LAS The total number of DMRG orbitals
+             \param one_dm The CASSCF 1-RDM
+             \param two_dm The CASSCF 2-RDM */
+         static void setDMRG1DM( const int num_elec, const int LAS, double * one_dm, double * two_dm );
+
+         //! Copy a one-orbital quantity from array format to DMRGSCFmatrix format
+         /** \param origin Array to copy
+             \param result DMRGSCFmatrix to store the copy
+             \param idx Object which handles the index conventions for CASSCF
+             \param one_rdm If true, the occupied orbitals get occupation 2 */
+         static void copy_active( double * origin, DMRGSCFmatrix * result, const DMRGSCFindices * idx, const bool one_rdm );
+
+         //! Copy a one-orbital quantity from DMRGSCFmatrix format to array format
+         /** \param origin DMRGSCFmatrix to copy
+             \param result Array to store the copy
+             \param idx Object which handles the index conventions for CASSCF */
+         static void copy_active( const DMRGSCFmatrix * origin, double * result, const DMRGSCFindices * idx );
+
          //! From an Edmiston-Ruedenberg active space rotation, fetch the eigenvectors and store them in eigenvecs
-         /** \param unitary The Edmiston-Ruedenberg active space rotation
-             \param localIdx Object which handles the index conventions for CASSCF
+         /** \param umat The Edmiston-Ruedenberg active space rotation
+             \param idx Object which handles the index conventions for CASSCF
              \param eigenvecs Where the eigenvectors are stored */
-         static void fillLocalizedOrbitalRotations(CheMPS2::DMRGSCFunitary * unitary, CheMPS2::DMRGSCFindices * localIdx, double * eigenvecs);
-         
+         static void fillLocalizedOrbitalRotations( DMRGSCFunitary * umat, DMRGSCFindices * idx, double * eigenvecs );
+
+         //! Block-diagonalize Mat
+         /** \param space Can be 'O', 'A', or 'V' and denotes which block of Mat should be considered
+             \param Mat Matrix to block-diagonalize
+             \param Umat The unitary rotation will be updated so that Mat is block-diagonal in the orbitals 'space'
+             \param work1 Workspace
+             \param work2 Workspace
+             \param idx Object which handles the index conventions for CASSCF
+             \param invert If true, the eigenvectors are sorted from large to small instead of the other way around
+             \param two_dm   If not NULL, this 4-index array will be rotated to the new eigenvecs if space == 'A'
+             \param three_dm If not NULL, this 6-index array will be rotated to the new eigenvecs if space == 'A'
+             \param contract If not NULL, this 6-index array will be rotated to the new eigenvecs if space == 'A' */
+         static void block_diagonalize( const char space, const DMRGSCFmatrix * Mat, DMRGSCFunitary * Umat, double * work1, double * work2, const DMRGSCFindices * idx, const bool invert, double * two_dm, double * three_dm, double * contract );
+
+         //! Construct the Fock matrix
+         /** \param Fock Matrix to store the Fock operator in
+             \param Tmat Matrix with the one-electron integrals
+             \param Qocc Matrix with the Coulomb and exchange contributions of the occupied (inactive) orbitals
+             \param Qact Matrix with the Coulomb and exchange contributions of the active space orbitals
+             \param idx Object which handles the index conventions for CASSCF */
+         static void construct_fock( DMRGSCFmatrix * Fock, const DMRGSCFmatrix * Tmat, const DMRGSCFmatrix * Qocc, const DMRGSCFmatrix * Qact, const DMRGSCFindices * idx );
+
+         //! Return the RMS deviation from block-diagonal
+         /** \param matrix Matrix to be assessed
+             \param idx Object which handles the index conventions for CASSCF
+             \return RMS deviation from block-diagonal */
+         static double deviation_from_blockdiag( DMRGSCFmatrix * matrix, const DMRGSCFindices * idx );
+
+         //! Write the checkpoint file for the contraction of the generalized Fock operator with the 4-RDM to disk
+         /** \param f4rdm_file The filename
+             \param hamorb1 The next hamiltonian orbital 1
+             \param hamorb2 The next hamiltonian orbital 2
+             \param tot_dmrg_power6 The size of the array contract
+             \param contract The current partial contraction */
+         static void write_f4rdm_checkpoint( const string f4rdm_file, int * hamorb1, int * hamorb2, const int tot_dmrg_power6, double * contract );
+
+         //! Read the checkpoint file for the contraction of the generalized Fock operator with the 4-RDM from disk
+         /** \param f4rdm_file The filename
+             \param hamorb1 The next hamiltonian orbital 1
+             \param hamorb2 The next hamiltonian orbital 2
+             \param tot_dmrg_power6 The size of the array contract
+             \param contract The current partial contraction
+             \return Whether the file was found and read */
+         static bool read_f4rdm_checkpoint( const string f4rdm_file, int * hamorb1, int * hamorb2, const int tot_dmrg_power6, double * contract );
+
       private:
-      
-         //Index convention handler
+
+         // CASSCF tmp folder
+         string tmp_folder;
+
+         // Index convention handler
          DMRGSCFindices * iHandler;
-         
-         //Unitary matrix storage and manipulator
+
+         // Unitary matrix storage and manipulator
          DMRGSCFunitary * unitary;
-         
-         //DIIS object
-         DIIS * theDIIS;
-      
-         //The original Hamiltonian
-         Hamiltonian * HamOrig;
-         bool shouldHamOrigBeDeleted;
-         
-         //The rotated 2-body matrix elements with at most two virtual indices
-         DMRGSCFintegrals * theRotatedTEI;
-         
-         //Irreps controller
+
+         // Whether CheMPS2::CASSCF::solve has been successfully terminated
+         bool successful_solve;
+
+         // The original Hamiltonian
+         double NUCL_ORIG;
+         const TwoIndex  * TMAT_ORIG;
+         const FourIndex * VMAT_ORIG;
+
+         // Irreps controller
          Irreps SymmInfo;
-         
-         //The number of orbitals
+
+         // The number of orbitals
          int L;
-         
-         //The numberOfIrreps;
-         int numberOfIrreps;
-         
-         //Double occupations
-         int * DOCC;
-         
-         //Single occupations
-         int * SOCC;
-         
-         //Boolean whether or not setupStart has been called
-         bool setupStartCalled;
-         
-         //Number of DMRG orbitals
+
+         // The number of irreps
+         int num_irreps;
+
+         // Number of DMRG orbitals
          int nOrbDMRG;
-         
-         //Space for the DMRG 1DM
+
+         // Space for the DMRG 1DM
          double * DMRG1DM;
 
-         //Space for the DMRG 2DM
+         // Space for the DMRG 2DM
          double * DMRG2DM;
-         
-         //Helper function to fetch DOCC and SOCC from filename
-         void allocateAndFillOCC(const string filename);
-         
-         //Helper function to copy the DOCC and SOCC arrays
-         void allocateAndFillOCC(int * DOCCin, int * SOCCin);
-         
-         //Helper function to check HF
-         void checkHF();
-         
-         //Fill Econst and Tmat of HamDMRG
-         void fillConstAndTmatDMRG(Hamiltonian * HamDMRG) const;
-         
-         //Calculate the gradient, return function is the gradient 2-norm
-         static double calcGradient(const DMRGSCFmatrix * localFmat, const DMRGSCFindices * localIdx, const DMRGSCFunitary * localUmat, double * gradient);
-         
-         //Calculate the hessian
-         static void calcHessian(const DMRGSCFmatrix * localFmat, const DMRGSCFwtilde * localwtilde, const DMRGSCFindices * localIdx, const DMRGSCFunitary * localUmat, double * hessian, const int rowjump);
-         
-         //Fmat function as defined by Eq. (11) in the Siegbahn paper.
+
+         // Helper function to check HF
+         void checkHF( int * docc, int * socc );
+
+         // Fill Econst and Tmat of HamDMRG
+         void fillConstAndTmatDMRG( Hamiltonian * HamDMRG ) const;
+
+         // Calculate the gradient, return function is the gradient 2-norm
+         static double construct_gradient( DMRGSCFmatrix * Fmatrix, const DMRGSCFindices * idx, double * gradient );
+
+         // Add hessian * origin to target
+         static void add_hessian( DMRGSCFmatrix * Fmatrix, DMRGSCFwtilde * Wtilde, const DMRGSCFindices * idx, double * origin, double * target );
+
+         // Construct the diagonal of the hessian
+         static void diag_hessian( DMRGSCFmatrix * Fmatrix, const DMRGSCFwtilde * Wtilde, const DMRGSCFindices * idx, double * diagonal );
+
+         // Apply augmented hessian
+         static void DGEMM_WRAP( double prefactor, char transA, char transB, double * A, double * B, double * C, int m, int n, int k, int lda, int ldb, int ldc );
+         static void DGEMV_WRAP( double prefactor, double * matrix, double * result, double * vector, int rowdim, int coldim, int ldmat, int incres, int incvec );
+         static void augmented_hessian( DMRGSCFmatrix * Fmatrix, DMRGSCFwtilde * Wtilde, const DMRGSCFindices * idx, double * origin, double * target, double * gradient, const int linsize );
+
+         // Rotate an active space object
+         static void rotate_active_space_object( const int num_indices, double * object, double * work, double * rotation, const int LAS, const int NJUMP, const int NROTATE );
+
+         // Fmat function as defined by Eq. (11) in the Siegbahn paper.
          DMRGSCFmatrix * theFmatrix;
-         
-         //The Coulomb and exchange interaction with the occupied and active electrons respectively
+
+         // The Coulomb and exchange interaction with the occupied and active electrons respectively
          DMRGSCFmatrix * theQmatOCC;
          DMRGSCFmatrix * theQmatACT;
          DMRGSCFmatrix * theQmatWORK;
          DMRGSCFmatrix * theTmatrix;
          void rotateOldToNew(DMRGSCFmatrix * myMatrix);
          void buildTmatrix();
-         void constructCoulombAndExchangeMatrixInOrigIndices(DMRGSCFmatrix * densityMatrix, DMRGSCFmatrix * resultMatrix);
+         void constructCoulombAndExchangeMatrixInOrigIndices( DMRGSCFmatrix * density, DMRGSCFmatrix * result );
          void buildQmatOCC();
          void buildQmatACT();
-         
-         //The Wmat_tilde function as defined by Eq. (20b) in the Siegbahn paper (see class header for specific definition)
-         DMRGSCFwtilde * wmattilde;
-         
-         //The Wmat function as defined by Eq. (21b) in the Siegbahn paper
-         static double Wmat(const DMRGSCFmatrix * localFmat, const DMRGSCFwtilde * localwtilde, const DMRGSCFindices * localIdx, const int irrep_pq, const int irrep_rs, const int relindexP, const int relindexQ, const int relindexR, const int relindexS);
-         
-         //Function to get the occupancies to obtain coefficients of certain Slater determinants for neutral C2. Important to figure out diatomic D(inf)h symmetries when calculating them in D2h symmetry. The function is not basis set and active space dependent (at least if no B2g, B3g, B2u and B3u orbitals are condensed).
-         void PrintCoeff_C2(DMRG * theDMRG);
-         
+
+         // Function to get coefficients of certain Slater determinants for Fe2. Important to figure out diatomic D(inf)h symmetries when calculating them in D2h symmetry.
+         void coeff_fe2( DMRG * theDMRG );
+
+         static void delete_file( const string filename );
+
    };
 }
 
